from panoramaTool.panorama_api.api_request import APICall
from panoramaTool.panorama_api.post_rules import PostRulesManager
from panoramaTool.panorama_api.service import ServiceManager

__version__ = '0.0.10'



import json

import requests


class ServiceManager:
    def __init__(self, panorama_url, apikey, verify=True):
        self.panorama_url = panorama_url
        self.apikey = apikey
        self.verify = verify
        self.url = f"{self.panorama_url}/restapi/v10.2/Objects/Services"
        self.headers = {"Content-Type": "application/json",
                        'X-PAN-KEY': f"{self.apikey}"}

    def list_services(self):
        params = {'location': 'shared', 'device-group': 'TestGroup'}
        response = requests.get(url=self.url, headers=self.headers, params=params, verify=self.verify)
        services = json.loads(response.content.decode('utf-8'))
        return services

    def create_service(self, protocol, port):
        params = {'name': f"{protocol}_{port}", 'location': 'shared', 'device-group': 'TestGroup'}
        payload = self._get_service_payload(protocol=protocol, port=port)
        response = requests.post(url=self.url, headers=self.headers, params=params, data=json.dumps(payload),
                                 verify=self.verify)
        service = json.loads(response.content.decode('utf-8'))
        return service

    def _get_service_payload(self, protocol, port):
        service_payload = {
            "entry": {
                "@name": f"{protocol}_{port}",
                "description": f"{protocol}_{port}",
                "protocol": {
                    protocol.lower(): {
                        "port": port
                    },
                },
            },
        }
        return service_payload



import concurrent.futures
import os

import csv

from flask import Flask, render_template, request
from dotenv import load_dotenv

from panoramaTool.panorama_api.api_request import APICall
from panoramaTool.panorama_api.post_rules import PostRulesManager

load_dotenv()

app = Flask(__name__)
template_path = os.path.join(os.path.dirname(__file__), 'templates')
app.template_folder = template_path
app.static_folder = os.path.join(os.path.dirname(__file__), 'static')


@app.route('/', methods=['GET', 'POST'])
def index():
    if request.method == 'GET':
        return render_template('form.html')
    elif request.method == 'POST':
        panorama_url = request.form['url']
        username = request.form['username']
        password = request.form['password']
        rule_name = request.form['rule_name']
        csv_file = request.files['csv_file']

        save_path = "panoramaTool/csv/upload.csv"

        if os.path.exists(save_path):
            os.remove(save_path)

        csv_file.save(save_path)
        apikey = APICall.get_api_key(user=username, password=password, panorama_url=panorama_url)
        post_rules_manager = PostRulesManager(panorama_url=panorama_url, apikey=apikey, verify=False)
        response = create_post_rules_from_csv(read_csv(save_path), post_rules_manager, rule_name=rule_name)
        print(response)
        return "Done!"


def create_post_rules_from_csv(csv, post_rules_manager, rule_name):
    response = ["No CSV"]

    with concurrent.futures.ThreadPoolExecutor() as executor:
        futures = []
        for counter, csv_data in enumerate(csv):
            futures.append(executor.submit(create_post_rule, csv_data, post_rules_manager, rule_name, counter))

        concurrent.futures.wait(futures)

        for future in futures:
            response.append(future.result())

    return response


def create_post_rule(csv_data, post_rules_manager, rule_name, counter):
    return post_rules_manager.create_post_rule(action=csv_data.get('Action'),
                                               source_address=csv_data.get('Source address'),
                                               source_zone=csv_data.get('Source Zone'),
                                               destination_address=csv_data.get('Destination address'),
                                               destination_zone=csv_data.get('Destination Zone'),
                                               protocol=csv_data.get('IP Protocol').upper(),
                                               destination_port=csv_data.get('Destination Port'),
                                               name=f"{rule_name}{counter}",
                                               application="any")


def read_csv(file_path):
    csv_data = []
    with open(file_path, 'r', encoding='utf-8-sig') as file:
        reader = csv.DictReader(file, delimiter=';')
        for row in reader:
            csv_data.append(row)
    return csv_data


if __name__ == '__main__':
    app.run()



import os
from xml.etree import ElementTree

import requests


class APICall:

    @staticmethod
    def get_api_key(panorama_url, user, password):
        user = user
        password = password
        key_response = requests.get(url=f"{panorama_url}/api/?type=keygen&user={user}&password={password}",
                                    verify=False)
        xml_key = key_response.content.decode('utf-8')
        root = ElementTree.fromstring(xml_key)
        apikey = root.find(".//key").text
        return apikey



# def create_zone_if_not_exist(panorama_tool_url, apikey, zone):
#     if zone not in get_zones(panorama_tool_url, apikey):
#         url = f"{panorama_url}/restapi/v10.2/Network/Zones"
#         params = {'name': zone, 'location': 'template', 'template': 'TestTemplate', 'vsys': "test"}
#         payload = {'name': zone, 'location': 'template', 'template': 'TestTemplate'}
#         headers = {"Content-Type": "application/json",
#                    'X-PAN-KEY': f"{apikey}"}
#         response = requests.post(url=url, headers=headers, params=params, data=json.dumps(payload), verify=False)
#         zones = json.loads(response.content.decode('utf-8'))
#         return zones
#     else:
#         return "Already exists."
#
# def get_zones(panorama_url, apikey):
#     url = f"{panorama_url}/restapi/v10.2/Network/Zones"
#     params = {'location': 'template', 'template': 'TestTemplate', 'vsys': "vsys1"}
#     headers = {"Content-Type": "application/json",
#                'X-PAN-KEY': f"{apikey}"}
#     response = requests.get(url=url, headers=headers, params=params, verify=False)
#     zones = json.loads(response.content.decode('utf-8'))
#     return zones
# def get_zone_payload(zone_name, zone_type, description=""):
#     zone_payload = {
#         "name": zone_name,
#         "zone_type": zone_type,
#         "description": description
#     }
#     return zone_payload

# if __name__ == "__main__":
    # response = get_zones(panorama_url=panorama_url, apikey=apikey)
    # response = create_zone_if_not_exist(panorama_url=panorama_url, apikey=apikey, zone="Zone_11")
    # response = create_zone_if_not_exist(panorama_url=panorama_url, apikey=apikey, zone="Transfer")
